@@ -10,9 +10,9 @@ use Plack::Util;
 use Class::Inspector;
 use List::Util qw(any);
 use Scalar::Util qw(blessed);
-use Encode qw(encode decode);
+use Kelp::Context;
 
-our $VERSION = '2.01_05';
+our $VERSION = '2.10_01';
 
 # Basic attributes
 attr -host => hostname;
@@ -28,6 +28,7 @@ attr long_error => $ENV{KELP_LONG_ERROR} // 0;
 # The charset is set to UTF-8 by default in config module.
 # No default here because we want to support 'undef' charset
 attr charset => sub { $_[0]->config('charset') };
+attr request_charset => sub { $_[0]->config('request_charset') };
 
 # Name the config module
 attr config_module => 'Config';
@@ -38,8 +39,8 @@ attr __config => undef;
 
 attr -loaded_modules => sub { {} };
 
-# Each route's request an response objects will
-# be put here:
+# Data for handling routes
+attr context => sub { Kelp::Context->new(app => $_[0]) };
 attr req => undef;
 attr res => undef;
 
@@ -51,15 +52,21 @@ sub new
 {
     my $self = shift->SUPER::new(@_);
 
+    Kelp::Base::_DEBUG(1 => 'Loading essential modules...');
+
     # Always load these modules, but allow client to override
     $self->_load_config();
     $self->_load_routes();
 
+    Kelp::Base::_DEBUG(1 => 'Loading modules from config...');
+
     # Load the modules from the config
     if (defined(my $modules = $self->config('modules'))) {
         $self->load_module($_) for (@$modules);
     }
 
+    Kelp::Base::_DEBUG(1 => 'Calling build method...');
+
     $self->build();
     return $self;
 }
@@ -104,6 +111,8 @@ sub _load_config
 {
     my $self = shift;
     $self->load_module($self->config_module, extra => $self->__config);
+
+    Kelp::Base::_DEBUG(config => 'Merged configuration: ', $self->config_hash);
 }
 
 sub _load_routes
@@ -146,6 +155,8 @@ sub load_module
         $args_from_config = $self->config("modules_init.$name") // {};
     }
 
+    Kelp::Base::_DEBUG(modules => "Loading $class module with args: ", {%$args_from_config, %args});
+
     $module->build(%$args_from_config, %args);
     return $module;
 }
@@ -158,19 +169,18 @@ sub build
 # Override to use a custom request object
 sub build_request
 {
-    my ($self, $env) = @_;
-    my $package = $self->request_obj;
-    eval qq{require $package};
-    return $package->new(app => $self, env => $env);
+    return Kelp::Util::load_package($_[0]->request_obj)->new(
+        app => $_[0],
+        env => $_[1],
+    );
 }
 
 # Override to use a custom response object
 sub build_response
 {
-    my $self = shift;
-    my $package = $self->response_obj;
-    eval qq{require $package};
-    return $package->new(app => $self);
+    return Kelp::Util::load_package($_[0]->response_obj)->new(
+        app => $_[0],
+    );
 }
 
 # Override to change what happens before the route is handled
@@ -204,6 +214,8 @@ sub run
     my $self = shift;
     my $app = sub { $self->psgi(@_) };
 
+    Kelp::Base::_DEBUG(1 => 'Running the application...');
+
     # Add middleware
     if (defined(my $middleware = $self->config('middleware'))) {
         for my $class (@$middleware) {
@@ -215,6 +227,9 @@ sub run
 
             my $mw = Plack::Util::load_class($class, 'Plack::Middleware');
             my $args = $self->config("middleware_init.$class") // {};
+
+            Kelp::Base::_DEBUG(modules => "Wrapping app in $mw middleware with args: ", $args);
+
             $app = $mw->wrap($app, %$args);
         }
     }
@@ -226,14 +241,15 @@ sub psgi
 {
     my ($self, $env) = @_;
 
-    # Create the request and response objects
+    # Initialize the app object state
+    $self->context->clear;
     my $req = $self->req($self->build_request($env));
     my $res = $self->res($self->build_response);
 
     # Get route matches
     my $match = $self->routes->match($req->path, $req->method);
 
-    # None found? Show 404 ...
+    # None found? Short-circuit and show 404
     if (!@$match) {
         $res->render_404;
         return $self->finalize;
@@ -325,8 +341,12 @@ sub psgi
 sub finalize
 {
     my $self = shift;
-    $self->before_finalize;
-    $self->res->finalize;
+
+    # call it with current context, so that it will get controller's hook if
+    # possible
+    $self->context->current->before_finalize;
+
+    return $self->res->finalize;
 }
 
 #----------------------------------------------------------------
@@ -380,20 +400,6 @@ sub abs_url
     return URI->new_abs($url, $self->config('app_url'))->as_string;
 }
 
-sub charset_encode
-{
-    my ($self, $string) = @_;
-    return $string unless $self->charset;
-    return encode $self->charset, $string;
-}
-
-sub charset_decode
-{
-    my ($self, $string) = @_;
-    return $string unless $self->charset;
-    return decode $self->charset, $string;
-}
-
 sub get_encoder
 {
     my ($self, $type, $name) = @_;
@@ -576,12 +582,23 @@ class will be used.
 
 =head2 charset
 
-Gets or sets the encoding charset of the app. It will be C<UTF-8>, if not set
-to anything else. The charset can also changed in the config files.
+Gets or sets the output encoding charset of the app. It will be C<UTF-8>, if
+not set to anything else. The charset can also changed in the config files.
 
 If the charset is explicitly configured to be C<undef> or false, the
-application won't do any automatic encoding of responses or decoding of
-requests (unless a request defines its own charset).
+application won't do any automatic encoding of responses, unless you set it by
+explicitly calling L<Kelp::Response/charset>.
+
+=head2 request_charset
+
+Same as L</charset>, but only applies to the input. Request data will be
+decoded using this charset or charset which came with the request.
+
+If the request charset is explicitly configured to be C<undef> or false, the
+application won't do any automatic decoding of requests, B<even if message came
+with a charset>.
+
+For details, see L<Kelp::Request/ENCODING>.
 
 =head2 long_error
 
@@ -612,6 +629,19 @@ contain a reference to the current L<Kelp::Response> instance.
         $self->res->json->render( { success => 1 } );
     }
 
+=head2 context
+
+This holds application's context. Its usage is advanced and only useful for
+controller logic, but may allow for some introspection into Kelp.
+
+For example, if you have a route in a controller and need to get the original
+Kelp app object, you may call this:
+
+    sub some_route {
+        my $controller = shift;
+        my $app = $controller->context->app;
+    }
+
 =head2 encoder_modules
 
 A hash reference of registered encoder modules. Should only be interacted with
@@ -799,12 +829,6 @@ arguments.
 Same as L</url_for>, but returns the full absolute URI for the current
 application (based on configuration).
 
-=head2 charset_encode
-
-=head2 charset_decode
-
-Shortcut methods, which encode or decode a string using the application's current L</charset>.
-
 =head2 is_production
 
 Returns whether the application is in production mode. Checks if L</mode> is
@@ -828,6 +852,16 @@ It will get extra config (if available) from C<encoders.TYPE.NAME>
 configuration hash. Will instantiate the encoder just once and then reuse it.
 Croaks when there is no such encoder type.
 
+Example new JSON encoder type defined in config:
+
+    encoders => {
+        json => {
+            not_very_pretty => {
+                pretty => 0,
+            },
+        },
+    },
+
 =head1 AUTHOR
 
 Stefan Geneshky - minimal <at> cpan.org
