@@ -2,25 +2,44 @@ package Kelp::Util;
 
 use Kelp::Base -strict;
 use Carp;
-use Scalar::Util qw(blessed);
+use Encode qw();
+
+# improve error locations of croak
+our @CARP_NOT = (qw(
+    Kelp
+    Kelp::Routes
+    Kelp::Context
+));
 
 sub camelize
 {
-    my ($string, $base) = @_;
+    my ($string, $base, $class_only) = @_;
     return $string unless $string;
 
     my $sigil = defined $string && $string =~ s/^(\+)// ? $1 : undef;
     $base = undef if $sigil;
 
-    my @parts = split(/\#/, $string);
-    my $sub = pop @parts;
+    my @parts;
+    if ($string =~ /::/) {
+        @parts = ($string);
+    }
+    else {
+        @parts = split /\#/, $string;
+        my $sub = pop @parts;
+
+        push @parts, $sub
+            if $class_only;
+
+        @parts = map {
+            join '', map { ucfirst lc } split /\_/
+        } @parts;
+
+        push @parts, $sub
+            if !$class_only;
+    }
 
-    @parts = map {
-        join '', map { ucfirst lc } split /\_/
-    } @parts;
     unshift @parts, $base if $base;
-
-    return join('::', @parts, $sub);
+    return join('::', @parts);
 }
 
 sub extract_class
@@ -47,6 +66,28 @@ sub extract_function
     return $string;
 }
 
+sub effective_charset
+{
+    my $this_charset = shift;
+    return Encode::find_encoding($this_charset) ? $this_charset : undef;
+}
+
+sub charset_encode
+{
+    my ($charset, $string) = @_;
+
+    return $string unless $charset;
+    return Encode::encode $charset, $string;
+}
+
+sub charset_decode
+{
+    my ($charset, $string) = @_;
+
+    return $string unless $charset;
+    return Encode::decode $charset, $string;
+}
+
 sub adapt_psgi
 {
     my ($app) = @_;
@@ -56,7 +97,7 @@ sub adapt_psgi
 
     return sub {
         my $kelp = shift;
-        my $path = $kelp->req->charset_encode(pop() // '');
+        my $path = charset_encode($kelp->request_charset, pop() // '');
         my $env = $kelp->req->env;
 
         # remember script and path
@@ -99,6 +140,18 @@ sub adapt_psgi
     };
 }
 
+sub load_package
+{
+    my $package = shift;
+    state $loaded = {};
+
+    # only string eval once for a given class name
+    return $loaded->{$package} //= do {
+        eval qq{require $package; 1} or croak $@;
+        $package;
+    };
+}
+
 1;
 
 __END__
@@ -146,7 +199,8 @@ This function accepts a string and a base class. Does three things:
 =back
 
 The returned string will have leading C<+> removed and will be prepended with
-the second argument if there was no C<+>.
+the second argument if there was no C<+>. An optional third argument can also
+be passed to treat the entire string as a class name.
 
 =head2 extract_class
 
@@ -158,6 +212,11 @@ no class in the string or the class is C<main>.
 Extracts the function name from a string. If there is no class name, returns
 the entire string. Returns undef for empty strings.
 
+=head2 effective_charset
+
+Takes a charset name and returns it back if it is supported by Encode.
+If there is no charset or it isn't supported, undef will be returned.
+
 =head2 adapt_psgi
 
 Transforms a given Plack/PSGI application (in form of a runner subroutine) to a
@@ -173,5 +232,9 @@ NOTE: having more than one placeholder in the pattern is mostly wasteful, as
 their matched values will not be handled in any way (other than allowing a
 varying request path).
 
+=head2 load_package
+
+Takes a name of a package and efficiently loads it.
+
 =cut
 
